@@ -771,10 +771,11 @@ c_fact(gdouble arg1)
 		return 0;
 	}
 	i = (int)arg1;
-	if(arg1!=i) {
+        if(fabs(arg1-i)<1e-9) {
 		errno=ERANGE;
 		return 0;
 	}
+        i = (int)(arg1+0.5);
 	for(r=1;i>0;i--)
 		r*=i;
 
