@@ -186,6 +186,14 @@ static void vt_line_update(struct _vtx *vx, struct vt_line *l, int line, int alw
 
       end--;
     }
+
+    /* always update the end if its a space ... */
+    if (end<l->width
+	&& ( (ch = l->data[end]&VTATTR_DATAMASK)==0
+	     || ch==9
+	     || ch==' '))
+      end++;
+
   }  /* else, update everything */
 
 
@@ -809,8 +817,8 @@ void vt_fix_selection(struct _vtx *vx)
 	     (( (vt_in_wordclass(vx, s->data[sx])))))
 	sx--;
 
-      if (sx &&
-	  (!vt_in_wordclass(vx, s->data[sx]&0xff)) )
+      if ((s->data[sx]&0xff)
+	  && (!vt_in_wordclass(vx, s->data[sx]&0xff)) )
 	sx++;
     }
     d(printf("%d\n", sx));
@@ -846,7 +854,8 @@ void vt_fix_selection(struct _vtx *vx)
       while ((ex<e->width) && ((e->data[ex]&0xff) == 0))
 	ex++;
   }
-  
+
+#if 1
   if ( ((vx->selstarty == vx->selendy) && (vx->selstartx > vx->selendx)) ||
        (vx->selstarty > vx->selendy) ) {
     vx->selstartx = ex;		/* swap end/start values */
@@ -855,6 +864,18 @@ void vt_fix_selection(struct _vtx *vx)
     vx->selstartx = sx;		/* swap end/start values */
     vx->selendx = ex;
   }
+#else
+  if ( ((vx->selstarty == vx->selendy) && (vx->selstartx > vx->selendx)) ||
+       (vx->selstarty > vx->selendy) ) {
+    vx->selectiontype = (vx->selectiontype & ~(VT_SELTYPE_BYSTART|VT_SELTYPE_BYEND))|VT_SELTYPE_BYEND;
+  } else {
+    vx->selectiontype = (vx->selectiontype & ~(VT_SELTYPE_BYSTART|VT_SELTYPE_BYEND))|VT_SELTYPE_BYSTART;
+  }
+  vx->selstartx = sx;
+  vx->selstarty = sy;
+  vx->selendx = ex;
+  vx->selendy = ey;
+#endif
 
   d(printf("fixed selection, now    at (%d,%d) (%d,%d)\n", sx, sy, ex, ey));
 }
@@ -872,7 +893,8 @@ static char *vt_expand_line(struct vt_line *l, int start, int end, char *out)
 
   /* scan from the end of the line to the start, looking for the
      actual end of screen data on that line */
-  for(dataend=l->width-1;dataend>0;dataend--) {
+  for(dataend=l->width;dataend>-1;) {
+    dataend--;
     if (l->data[dataend] & 0xff) {
       dataend++;
       break;
@@ -1065,6 +1087,14 @@ void vt_draw_selection(struct _vtx *vx)
      difference between old start and new start
      difference between old end and new end
   */
+
+  /* dont draw nothin if we dont have to! */
+  if (vx->selendxold == vx->selstartxold
+      && vx->selendx == vx->selstartx
+      && vx->selendyold == vx->selstartyold
+      &&  vx->selendy == vx->selstarty)
+    return;
+
   d(printf("start ... \n"));
   vt_draw_selection_part(vx,
 			 vx->selstartx, vx->selstarty,
